@@ -67,6 +67,7 @@ def test_unit_in_name():
                     "type": "memory_distribution",
                     "memory_unit": "megabyte",
                 },
+                # This should be allowed.
                 "width_pixels": {
                     "type": "quantity",
                     "gecko_datapoint": "WIDTH_PIXELS",
@@ -83,7 +84,7 @@ def test_unit_in_name():
 
     nits = lint.lint_metrics(all_metrics.value)
 
-    assert len(nits) == 3
+    assert len(nits) == 2
     assert all(nit.check_name == "UNIT_IN_NAME" for nit in nits)
 
     # Now make sure the override works
@@ -94,7 +95,7 @@ def test_unit_in_name():
 
     nits = lint.lint_metrics(all_metrics.value)
 
-    assert len(nits) == 2
+    assert len(nits) == 1
 
 
 def test_category_generic():
@@ -135,7 +136,7 @@ def test_combined():
                 "m_width_pixels": {
                     "type": "quantity",
                     "gecko_datapoint": "WIDTH_PIXELS",
-                    "unit": "pixels",
+                    "unit": "pixels", # not a lint issue
                 },
             }
         }
@@ -148,7 +149,7 @@ def test_combined():
 
     nits = lint.lint_metrics(all_metrics.value)
 
-    assert len(nits) == 5
+    assert len(nits) == 4
     assert set(["COMMON_PREFIX", "CATEGORY_GENERIC", "UNIT_IN_NAME"]) == set(
         v.check_name for v in nits
     )
@@ -181,8 +182,16 @@ def test_misspelling_pings():
         {
             "user_data": {
                 "counter": {"type": "counter", "send_in_pings": ["metric"]},
-                "string": {"type": "string", "send_in_pings": ["event"]},
-                "string2": {"type": "string", "send_in_pings": ["metrics", "events"]},
+                "string": {
+                    "type": "string",
+                    "lifetime": "application",
+                    "send_in_pings": ["event"],
+                },
+                "string2": {
+                    "type": "string",
+                    "lifetime": "application",
+                    "send_in_pings": ["metrics", "events"],
+                },
             }
         }
     ]
@@ -198,25 +207,6 @@ def test_misspelling_pings():
     assert set(["MISSPELLED_PING"]) == set(v.check_name for v in nits)
 
 
-def test_yaml_lint(capsys):
-    """Tests yamllint on files with nits."""
-    file_paths = [ROOT / "data" / "core.yaml", ROOT / "data" / "yaml_nits.yamlx"]
-
-    nits = lint.lint_yaml_files(file_paths)
-
-    assert len(nits) == 3
-    # The second rule is empty because it's a syntax error.
-    assert set(["indentation", None, "trailing-spaces"]) == set(v.rule for v in nits)
-
-    captured = capsys.readouterr()
-    lines = captured.out.split("\n")
-    for line in lines:
-        if line.strip() == "":
-            continue
-        assert "yaml_nits.yamlx" in line
-        assert "core.yaml" not in line
-
-
 def test_user_lifetime_expiration():
     """Test that expiring 'user' lifetime metrics generate a warning."""
     contents = [
@@ -298,7 +288,29 @@ def test_expires_too_far_in_the_future():
     assert set(["EXPIRATION_DATE_TOO_FAR"]) == set(v.check_name for v in nits)
 
 
-def test_translate_missing_input_files(tmpdir):
+def test_invalid_lifetime_for_metric_on_events_ping():
+    """Test that a `ping` lifetime, non-event metric, fails when sent on the
+    Metrics ping"""
+    contents = [
+        {
+            "user_data": {
+                "invalid_lifetime": {
+                    "type": "counter",
+                    "lifetime": "ping",
+                    "send_in_pings": "events",
+                }
+            }
+        }
+    ]
+
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+
+    errs = list(all_metrics)
+    assert len(errs) == 1
+
+
+def test_translate_missing_input_files(tmp_path):
     with pytest.raises(FileNotFoundError):
         lint.glinter(
             [ROOT / "data" / "missing.yaml"],
@@ -480,3 +492,108 @@ def test_old_event_api():
     assert nits[0].check_name == "OLD_EVENT_API"
     assert nits[0].name == "old_event.name"
     assert "Extra keys require a type" in nits[0].msg
+
+
+def test_unknown_pings_lint():
+    """Test that the 'glinter' reports issues with unknown pings in send_in_pings."""
+    input = [ROOT / "data" / "unknown_ping_used.yaml", ROOT / "data" / "pings.yaml"]
+    all_objects = parser.parse_objects(input)
+
+    errs = list(all_objects)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_objects.value, parser_config={})
+    assert len(nits) == 2
+    assert nits[0].check_name == "UNKNOWN_PING_REFERENCED"
+    assert nits[0].name == "all_metrics.non_existent_ping"
+    assert "does-not-exist" in nits[0].msg
+
+
+def test_name_too_similar_lint():
+    """Ensure the 'glinter' reports metrics whose names are too similar."""
+    # Note: NAME_TOO_SIMILAR is an all-object lint meaning we need pings for it to work.
+    input = [ROOT / "data" / "name_too_similar.yaml", ROOT / "data" / "pings.yaml"]
+    all_objects = parser.parse_objects(input)
+
+    errs = list(all_objects)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_objects.value, parser_config={})
+    assert len(nits) == 1
+    assert nits[0].check_name == "NAME_TOO_SIMILAR"
+    assert nits[0].name == "all_metrics.validmetric"
+    assert "all_metrics.valid_metric" in nits[0].msg
+
+
+@pytest.mark.parametrize(
+    "metric, num_nits",
+    [
+        ({"metric": {"data_reviews": ["12345"]}}, 0),
+        ({"metric": {"data_reviews": ["12345", "", "TODO"]}}, 1),
+        ({"metric": {"data_reviews": [""]}}, 1),
+        ({"metric": {"data_reviews": [""], "no_lint": ["EMPTY_DATAREVIEW"]}}, 0),
+        ({"metric": {"data_reviews": ["TODO"]}}, 1),
+        ({"metric": {"data_reviews": ["TODO"], "no_lint": ["EMPTY_DATAREVIEW"]}}, 0),
+    ],
+)
+def test_empty_datareviews(metric, num_nits):
+    """
+    Test that the list of data reviews does not contain empty strings or TODO markers
+    """
+    content = {"category": metric}
+    content = util.add_required(content)
+    all_metrics = parser.parse_objects(content)
+
+    errs = list(all_metrics)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_metrics.value)
+
+    assert len(nits) == num_nits
+    if num_nits > 0:
+        assert set(["EMPTY_DATAREVIEW"]) == set(v.check_name for v in nits)
+
+
+@pytest.mark.parametrize(
+    "metric, num_nits",
+    [
+        ({"metric": {"type": "quantity", "unit": "sheep"}}, 0),
+        (
+            {
+                "metric": {
+                    "type": "custom_distribution",
+                    "unit": "quantillions",
+                    "range_max": 100,
+                    "bucket_count": 100,
+                    "histogram_type": "linear",
+                }
+            },
+            0,
+        ),
+        ({"metric": {"type": "string", "unit": "quantillions"}}, 1),
+        ({"metric": {"type": "counter", "unit": "quantillions"}}, 1),
+        (
+            {
+                "metric": {
+                    "type": "string",
+                    "unit": "quantillions",
+                    "no_lint": ["UNEXPECTED_UNIT"],
+                }
+            },
+            0,
+        ),
+    ],
+)
+def test_unit_on_metrics(metric, num_nits):
+    content = {"category": metric}
+    content = util.add_required(content)
+    all_metrics = parser.parse_objects(content)
+
+    errs = list(all_metrics)
+    assert len(errs) == 0
+
+    nits = lint.lint_metrics(all_metrics.value)
+
+    assert len(nits) == num_nits
+    if num_nits > 0:
+        assert set(["UNEXPECTED_UNIT"]) == set(v.check_name for v in nits)
