@@ -6,7 +6,6 @@
 from pathlib import Path
 import json
 import re
-import sys
 import textwrap
 
 import pytest
@@ -45,6 +44,10 @@ def test_parser():
     assert pings["really-custom-ping"].name == "really-custom-ping"
     assert pings["really-custom-ping"].include_client_id is True
 
+    # `follows_collection_enabled` forces `enabled=false`
+    assert pings["nofollows"].follows_collection_enabled is False
+    assert pings["nofollows"].enabled is False
+
 
 def test_parser_invalid():
     """Test the basics of parsing a single file."""
@@ -54,10 +57,6 @@ def test_parser_invalid():
     assert "could not determine a constructor for the tag" in errors[0]
 
 
-@pytest.mark.skipif(
-    sys.version_info < (3, 7),
-    reason="Error messages have literal 'OrderedDict' in them on Python 3.6",
-)
 def test_parser_schema_violation():
     """1507792"""
     all_metrics = parser.parse_objects(ROOT / "data" / "schema-violation.yaml")
@@ -115,13 +114,13 @@ def test_parser_schema_violation():
         """,
         """
         ```
-        gleantest.short.category:very_long_metric_name_this_is_too_long_s_well
+        gleantest.short.category:very_long_metric_name_this_is_too_long_as_well_since_it_has_sooooo_many_characters
         ```
 
-        'very_long_metric_name_this_is_too_long_s_well' is not valid under any
+        'very_long_metric_name_this_is_too_long_as_well_since_it_has_sooooo_many_characters' is not valid under any
         of the given schemas
-        'very_long_metric_name_this_is_too_long_s_well' is too long
-        """,
+        'very_long_metric_name_this_is_too_long_as_well_since_it_has_sooooo_many_characters' is too long
+        """,  # noqa: E501 #
         """
         ```
         gleantest:
@@ -190,6 +189,111 @@ def test_parser_schema_violation():
                 key_16:
                 description: Sample extra key
                 type: string
+                key_17:
+                description: Sample extra key
+                type: string
+                key_18:
+                description: Sample extra key
+                type: string
+                key_19:
+                description: Sample extra key
+                type: string
+                key_20:
+                description: Sample extra key
+                type: string
+                key_21:
+                description: Sample extra key
+                type: string
+                key_22:
+                description: Sample extra key
+                type: string
+                key_23:
+                description: Sample extra key
+                type: string
+                key_24:
+                description: Sample extra key
+                type: string
+                key_25:
+                description: Sample extra key
+                type: string
+                key_26:
+                description: Sample extra key
+                type: string
+                key_27:
+                description: Sample extra key
+                type: string
+                key_28:
+                description: Sample extra key
+                type: string
+                key_29:
+                description: Sample extra key
+                type: string
+                key_30:
+                description: Sample extra key
+                type: string
+                key_31:
+                description: Sample extra key
+                type: string
+                key_32:
+                description: Sample extra key
+                type: string
+                key_33:
+                description: Sample extra key
+                type: string
+                key_34:
+                description: Sample extra key
+                type: string
+                key_35:
+                description: Sample extra key
+                type: string
+                key_36:
+                description: Sample extra key
+                type: string
+                key_37:
+                description: Sample extra key
+                type: string
+                key_38:
+                description: Sample extra key
+                type: string
+                key_39:
+                description: Sample extra key
+                type: string
+                key_40:
+                description: Sample extra key
+                type: string
+                key_41:
+                description: Sample extra key
+                type: string
+                key_42:
+                description: Sample extra key
+                type: string
+                key_43:
+                description: Sample extra key
+                type: string
+                key_44:
+                description: Sample extra key
+                type: string
+                key_45:
+                description: Sample extra key
+                type: string
+                key_46:
+                description: Sample extra key
+                type: string
+                key_47:
+                description: Sample extra key
+                type: string
+                key_48:
+                description: Sample extra key
+                type: string
+                key_49:
+                description: Sample extra key
+                type: string
+                key_50:
+                description: Sample extra key
+                type: string
+                key_51:
+                description: Sample extra key
+                type: string
         ```
         {'key_1': {'description': 'Sample extra key','type': 'string'},
         'key_2': {'description': 'Sample extra key','type': 'string'},
@@ -206,12 +310,47 @@ def test_parser_schema_violation():
         'key_13': {'description': 'Sample extra key','type': 'string'},
         'key_14': {'description': 'Sample extra key','type': 'string'},
         'key_15': {'description': 'Sample extra key','type': 'string'},
-        'key_16': {'description': 'Sample extra key','type': 'string'}
+        'key_16': {'description': 'Sample extra key','type': 'string'},
+        'key_17': {'description': 'Sample extra key','type': 'string'},
+        'key_18': {'description': 'Sample extra key','type': 'string'},
+        'key_19': {'description': 'Sample extra key','type': 'string'},
+        'key_20': {'description': 'Sample extra key','type': 'string'},
+        'key_21': {'description': 'Sample extra key','type': 'string'},
+        'key_22': {'description': 'Sample extra key','type': 'string'},
+        'key_23': {'description': 'Sample extra key','type': 'string'},
+        'key_24': {'description': 'Sample extra key','type': 'string'},
+        'key_25': {'description': 'Sample extra key','type': 'string'},
+        'key_26': {'description': 'Sample extra key','type': 'string'},
+        'key_27': {'description': 'Sample extra key','type': 'string'},
+        'key_28': {'description': 'Sample extra key','type': 'string'},
+        'key_29': {'description': 'Sample extra key','type': 'string'},
+        'key_30': {'description': 'Sample extra key','type': 'string'},
+        'key_31': {'description': 'Sample extra key','type': 'string'},
+        'key_32': {'description': 'Sample extra key','type': 'string'},
+        'key_33': {'description': 'Sample extra key','type': 'string'},
+        'key_34': {'description': 'Sample extra key','type': 'string'},
+        'key_35': {'description': 'Sample extra key','type': 'string'},
+        'key_36': {'description': 'Sample extra key','type': 'string'},
+        'key_37': {'description': 'Sample extra key','type': 'string'},
+        'key_38': {'description': 'Sample extra key','type': 'string'},
+        'key_39': {'description': 'Sample extra key','type': 'string'},
+        'key_40': {'description': 'Sample extra key','type': 'string'},
+        'key_41': {'description': 'Sample extra key','type': 'string'},
+        'key_42': {'description': 'Sample extra key','type': 'string'},
+        'key_43': {'description': 'Sample extra key','type': 'string'},
+        'key_44': {'description': 'Sample extra key','type': 'string'},
+        'key_45': {'description': 'Sample extra key','type': 'string'},
+        'key_46': {'description': 'Sample extra key','type': 'string'},
+        'key_47': {'description': 'Sample extra key','type': 'string'},
+        'key_48': {'description': 'Sample extra key','type': 'string'},
+        'key_49': {'description': 'Sample extra key','type': 'string'},
+        'key_50': {'description': 'Sample extra key','type': 'string'},
+        'key_51': {'description': 'Sample extra key','type': 'string'}
         } has too many properties
         Documentation for this node:
             The acceptable keys on the "extra" object sent with events. This is an
             object mapping the key to an object containing metadata about the key.
-            A maximum of 15 extra keys is allowed.
+            A maximum of 50 extra keys is allowed.
             This metadata object has the following keys:
                 - `description`: **Required.** A description of the key.
             Valid when `type`_ is `event`.
@@ -434,7 +573,7 @@ def test_custom_distribution():
                     "type": "custom_distribution",
                     "range_min": 0,
                     "range_max": 60000,
-                    "bucket_count": 100,
+                    "bucket_count": 200,
                     "histogram_type": "exponential",
                 }
             }
@@ -452,7 +591,6 @@ def test_custom_distribution():
             "category": {
                 "metric": {
                     "type": "custom_distribution",
-                    "gecko_datapoint": "FROM_GECKO",
                 }
             }
         }
@@ -464,27 +602,6 @@ def test_custom_distribution():
     assert len(errors) == 1
     assert "`custom_distribution` is missing required parameters" in errors[0]
 
-    # Test maximum bucket_count is enforced
-    contents = [
-        {
-            "category": {
-                "metric": {
-                    "type": "custom_distribution",
-                    "gecko_datapoint": "FROM_GECKO",
-                    "range_max": 60000,
-                    "bucket_count": 101,
-                    "histogram_type": "exponential",
-                }
-            }
-        }
-    ]
-
-    contents = [util.add_required(x) for x in contents]
-    all_metrics = parser.parse_objects(contents)
-    errors = list(all_metrics)
-    assert len(errors) == 1
-    assert "101 is greater than" in errors[0]
-
     # Test that correct usage
     contents = [
         {
@@ -492,7 +609,7 @@ def test_custom_distribution():
                 "metric": {
                     "type": "custom_distribution",
                     "range_max": 60000,
-                    "bucket_count": 100,
+                    "bucket_count": 200,
                     "histogram_type": "exponential",
                 }
             }
@@ -506,7 +623,7 @@ def test_custom_distribution():
     distribution = all_metrics.value["category"]["metric"]
     assert distribution.range_min == 1
     assert distribution.range_max == 60000
-    assert distribution.bucket_count == 100
+    assert distribution.bucket_count == 200
     assert distribution.histogram_type == metrics.HistogramType.exponential
 
 
@@ -588,6 +705,40 @@ def test_do_not_disable_expired():
     assert metrics["category"]["metric"].disabled is False
 
 
+def test_interesting_disables_others():
+    # Test that if we get an "interesting" config param, only the metrics in
+    # the provided config files are enabled.
+
+    # We'll have aall_metrics.yaml available loaded.
+    all_metrics = ROOT / "data" / "all_metrics.yaml"
+    # But, only the metrics in metric-with-tags.yaml are marked as interesting
+    # so every other metric will be marked as disabled.
+    metrics_with_tags_yaml = ROOT / "data" / "metric-with-tags.yaml"
+    interesting = [Path(path) for path in [metrics_with_tags_yaml]]
+    config = {"interesting": interesting}
+
+    all_metrics = parser.parse_objects([all_metrics, metrics_with_tags_yaml], config)
+    errors = list(all_metrics)
+    assert len(errors) == 0
+
+    interesting_metrics = parser.parse_objects([metrics_with_tags_yaml])
+    errors = list(interesting_metrics)
+    assert len(errors) == 0
+
+    metrics = all_metrics.value
+    for category_key, category_val in metrics.items():
+        if category_key == "tags":
+            continue
+
+        for metric_key, metric_val in sorted(category_val.items()):
+            cat = interesting_metrics.value.get(category_key)
+            if cat and cat.get(metric_key):
+                disabled = cat.get(metric_key).disabled
+            else:
+                disabled = True
+            assert metrics[category_key][metric_key].disabled is disabled
+
+
 def test_send_in_pings_restrictions():
     """Test that invalid ping names are disallowed in `send_in_pings`."""
     all_metrics = parser.parse_objects(ROOT / "data" / "invalid-ping-names.yaml")
@@ -905,7 +1056,7 @@ def test_text_valid():
     assert all_metrics.value["valid.text"]["lifetime"].lifetime == metrics.Lifetime.ping
 
     assert all_metrics.value["valid.text"]["sensitivity"].data_sensitivity == [
-        metrics.DataSensitivity.web_activity
+        metrics.DataSensitivity.stored_content
     ]
 
 
@@ -1044,3 +1195,95 @@ def test_no_internal_fields_exposed():
         indent=2,
     )
     assert expected_json == out_json
+
+
+def test_object():
+    structure = {"type": "array", "items": {"type": "number"}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 0, errors
+    assert len(all_metrics.value) == 1
+    assert all_metrics.value["category"]["metric"]._generate_structure == structure
+
+
+def test_object_invalid():
+    contents = [{"category": {"metric": {"type": "object"}}}]
+
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "`object` is missing required parameter `structure`" in errors[0]
+
+    structure = {"type": "array", "items": {}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "missing `type`" in errors[0]
+    assert "Allowed: [" in errors[0]
+
+    structure = {"type": "array", "items": {"type": "unknown"}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "invalid `type`" in errors[0]
+    assert "found: unknown, allowed: [" in errors[0]
+
+    structure = {"type": "array", "properties": {}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "`properties` not allowed in array structure" in errors[0]
+
+    structure = {"type": "object", "items": {}}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "`items` not allowed in object structure" in errors[0]
+
+    structure = {"type": "string"}
+    contents = [{"category": {"metric": {"type": "object", "structure": structure}}}]
+    contents = [util.add_required(x) for x in contents]
+    all_metrics = parser.parse_objects(contents)
+    errors = list(all_metrics)
+    assert len(errors) == 1
+    assert "invalid `type` in object structure." in errors[0]
+
+
+def test_reserved_category():
+    all_metrics = parser.parse_objects(ROOT / "data" / "reserved_categories.yamlx")
+
+    errors = list(all_metrics)
+    assert len(errors) == 2
+    assert "Categories beginning with 'glean' are reserved" in errors[0]
+    assert "Categories beginning with 'glean' are reserved" in errors[1]
+
+
+def test_attribution_distribution():
+    all_metrics = parser.parse_objects(ROOT / "data" / "attribution.yaml")
+
+    errors = list(all_metrics)
+    assert len(errors) == 0
+
+
+def test_forbid_non_ext_non_object_attribution_distribution():
+    all_metrics = parser.parse_objects(ROOT / "data" / "bad_attribution.yamlx")
+
+    errors = list(all_metrics)
+    assert len(errors) == 2
+    assert "May only use semi-reserved category" in errors[0]
+    assert (
+        "Extended attribution/distribution metrics must be of type 'object'"
+        in errors[1]
+    )
